<?php
// $Id: sunmailer.admin.inc,v 1.1.2.1 2009/12/26 18:17:19 mwacker Exp $

/**
 * @file
 * Configures the settings for Sunmailer.
 */

/**
 * Generates a form for PHPMailer settings.
 */
function sunmailer_admin_settings() {
  $form = array();
  // Important settings.
  $form['sunmailer_phpmailer_path'] = array(
    '#type' => 'textfield',
    '#title' => t('PHPMailer Path'),
    '#description' => t('Absolute path to the directory containing class.phpmailer.php'),
    '#default_value' => variable_get('sunmailer_phpmailer_path', ''),
  );
  $form['sunmailer_test_email'] = array(
    '#type' => 'textfield',
    '#title' => t('Test Email'),
    '#description' => t('Email address to receive test emails'),
    '#default_value' => variable_get('sunmailer_test_email', ''),
  );
  $form['button'] = array(
    '#type' => 'submit',
    '#value' => t('Send Test Email'),
  );

  // Basic settings.
  $form['basic'] = array(
    '#type' => 'fieldset',
    '#title' => t('Basic Settings'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );
  $form['basic']['sunmailer_from_name'] = array(
    '#type' => 'textfield',
    '#title' => t('From Name'),
    '#description' => t('Name of the sender'),
    '#required' => TRUE,
    '#default_value' => variable_get('sunmailer_from_name', ''),
  );
  $form['basic']['sunmailer_from_email'] = array(
    '#type' => 'textfield',
    '#title' => t('From Email'),
    '#description' => t('Email of the sender'),
    '#required' => TRUE,
    '#default_value' => variable_get('sunmailer_from_email', ''),
  );
  $form['basic']['sunmailer_host'] = array(
    '#type' => 'textfield',
    '#title' => t('Host'),
    '#description' => t('URL or IP address for the server sending the message'),
    '#required' => TRUE,
    '#default_value' => variable_get('sunmailer_host', 'localhost'),
  );
  $form['basic']['sunmailer_port'] = array(
    '#type' => 'textfield',
    '#title' => t('Port'),
    '#description' => t('Port the server sends the message on'),
    '#required' => TRUE,
    '#default_value' => variable_get('sunmailer_port', 25),
  );

  // Mail settings.
  $form['mail'] = array(
    '#type' => 'fieldset',
    '#title' => t('Mail Settings'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );
  $form['mail']['sunmailer_method'] = array(
    '#type' => 'radios',
    '#title' => t('Mailing Method'),
    '#description' => t('Selects the method used to send the newsletter'),
    '#required' => TRUE,
    '#options' => array(
      'mail' => 'Mail',
      'sendmail' => 'Sendmail',
      'smtp' => 'SMTP',
    ),
    '#default_value' => variable_get('sunmailer_method', 'mail'),
  );
  // Sendmail settings.
  $form['mail']['sunmailer_sendmail_path'] = array(
    '#type' => 'textfield',
    '#title' => t('Sendmail Path'),
    '#description' => t('Absolute path to the sendmail program'),
    '#default_value' => variable_get('sunmailer_sendmail_path', '/usr/bin/sendmail'),
  );
  // SMTP settings.
  $form['mail']['sunmailer_smtp_auth'] = array(
    '#type' => 'checkbox',
    '#title' => t('SMTP Authentication'),
    '#description' => t('Check this to enable SMTP authentication'),
    '#default_value' => variable_get('sunmailer_smtp_auth', FALSE),
  );
  $form['mail']['sunmailer_smtp_user'] = array(
    '#type' => 'textfield',
    '#title' => t('SMTP Username'),
    '#description' => t('Username for SMTP authentication'),
    '#default_value' => variable_get('sunmailer_smtp_user', ''),
  );
  $form['mail']['sunmailer_smtp_pass'] = array(
    '#type' => 'textfield',
    '#title' => t('SMTP Password'),
    '#description' => t('Password for SMTP authentication'),
    '#default_value' => variable_get('sunmailer_smtp_pass', ''),
  );
  $form['mail']['sunmailer_smtp_secure'] = array(
    '#type' => 'radios',
    '#title' => t('SMTP Security'),
    '#description' => t('Sets the encryption used for sending emails'),
    '#options' => array(
      '' => t('None'),
      'ssl' => 'SSL',
      'tls' => 'TLS',
    ),
    '#default_value' => variable_get('sunmailer_smtp_secure', ''),
  );

  // Advanced settings.
  $form['advanced'] = array(
    '#type' => 'fieldset',
    '#title' => t('Advanced Settings'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );
  $form['advanced']['sunmailer_delay'] = array(
    '#type' => 'textfield',
    '#title' => t('Next Mailing Delay'),
    '#description' => t('Number of minutes to wait before allowing another mailing'),
    '#required' => TRUE,
    '#default_value' => variable_get('sunmailer_delay', 5),
  );
  $form['advanced']['sunmailer_retries'] = array(
    '#type' => 'textfield',
    '#title' => t('Resend Attempts'),
    '#description' => t('Number of times to retry if the message fails to be sent'),
    '#required' => TRUE,
    '#default_value' => variable_get('sunmailer_retries', 2),
  );
  $form['advanced']['sunmailer_max_recipients'] = array(
    '#type' => 'textfield',
    '#title' => t('Maximum Number of Recipients'),
    '#description' => t('Emails with more than the maximum will be split into multiple emails.  Use 0 for no limit.'),
    '#required' => TRUE,
    '#default_value' => variable_get('sunmailer_max_recipients', 0),
  );
  $form['advanced']['sunmailer_word_wrap'] = array(
    '#type' => 'textfield',
    '#title' => t('Word Wrap'),
    '#description' => t('Maximum number of text characters on one line'),
    '#required' => TRUE,
    '#default_value' => variable_get('sunmailer_word_wrap', 80),
  );
  $form['advanced']['sunmailer_char_set'] = array(
    '#type' => 'textfield',
    '#title' => t('Character Set'),
    '#description' => t('Character encoding to use for emails'),
    '#required' => TRUE,
    '#default_value' => 'utf-8',
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Submit'),
  );
  return $form;
}

/**
 * Validation callback for PHPMailer settings form.
 */
function sunmailer_admin_settings_validate($form, &$form_state) {
  $values = $form_state['values'];
  // Validate top-level settings and email test button.
  if (!file_exists("{$values['sunmailer_phpmailer_path']}/class.phpmailer.php")) {
    form_set_error('sunmailer_phpmailer_path', t('Cannot find PHPMailer.'));
  }
  if ($form_state['clicked_button']['#value'] == 'Send Test Email') {
    if (!$values['sunmailer_test_email']) {
      form_set_error('sunmailer_test_email', t('Test Email is required to send a test email.'));
    }
  }
  // Validate basic settings.
  if (!is_numeric($values['sunmailer_port']) || $values['sunmailer_port'] < 0) {
    form_set_error('sunmailer_port', t('Port must be a positive integer.'));
  }
  // Validate mail settings.
  switch ($values['sunmailer_method']) {
  case 'sendmail':
    if (!$values['summailer_sendmail_path']) {
      form_set_error('sunmailer_sendmail_path', t('Sendmail Path is required if using Sendmail.'));
    }
    break;
  case 'smtp':
    if ($values['sunmailer_smtp_auth']) {
      if (!$values['sunmailer_smtp_user']) {
        form_set_error('sunmailer_smtp_user', t('SMTP Username is required if using SMTP with authentication.'));
      }
      if (!$values['sunmailer_smtp_pass']) {
        form_set_error('sunmailer_smtp_pass', t('SMTP Password is required if using SMTP with authentication.'));
      }
    }
    break;
  }
  // Validate advanced settings.
  if (!is_numeric($values['sunmailer_delay']) || $values['sunmailer_delay'] < 0) {
    form_set_error('sunmailer_delay', t('Next Mailing Delay must be a positive integer.'));
  }
  if (!is_numeric($values['sunmailer_retries']) || $values['sunmailer_retries'] < 0) {
    form_set_error('sunmailer_retries', t('Resend Attempts must be a positive integer.'));
  }
  if (!is_numeric($values['sunmailer_max_recipients']) || $values['sunmailer_max_recipients'] < 0) {
    form_set_error('sunmailer_max_recipients', t('Maximum Number of Recipients must be a positive integer.'));
  }
  if (!is_numeric($values['sunmailer_word_wrap']) || $values['sunmailer_word_wrap'] < 0) {
    form_set_error('sunmailer_word_wrap', t('Word Wrap must be a positive integer.'));
  }
}

/**
 * Submission callback for PHPMailer settings form.
 */
function sunmailer_admin_settings_submit($form, &$form_state) {
  $values = $form_state['values'];
  // Set top-level settings.
  variable_set('sunmailer_phpmailer_path', $values['sunmailer_phpmailer_path']);
  variable_set('sunmailer_test_email', $values['sunmailer_test_email']);
  // Set basic settings.
  variable_set('sunmailer_from_name', $values['sunmailer_from_name']);
  variable_set('sunmailer_from_email', $values['sunmailer_from_email']);
  variable_set('sunmailer_host', $values['sunmailer_host']);
  variable_set('sunmailer_port', $values['sunmailer_port']);
  // Set mail settings.
  variable_set('sunmailer_method', $values['sunmailer_method']);
  variable_set('sunmailer_sendmail_path', $values['sunmailer_sendmail_path']);
  variable_set('sunmailer_smtp_auth', $values['sunmailer_smtp_auth']);
  variable_set('sunmailer_smtp_user', $values['sunmailer_smtp_user']);
  variable_set('sunmailer_smtp_pass', $values['sunmailer_smtp_pass']);
  variable_set('sunmailer_smtp_secure', $values['sunmailer_smtp_secure']);
  // Set advanced settings.
  variable_set('sunmailer_delay', $values['sunmailer_delay']);
  variable_set('sunmailer_retries', $values['sunmailer_retries']);
  variable_set('sunmailer_max_recipients', $values['sunmailer_max_recipients']);
  variable_set('sunmailer_word_wrap', $values['sunmailer_word_wrap']);
  variable_set('sunmailer_char_set', $values['sunmailer_char_set']);
  // Send a test email if requested.
  if ($form_state['clicked_button']['#value'] == 'Send Test Email') {
    $mailer = sunmailer_get_mailer();
    $mailer->AddAddress(variable_get('sunmailer_test_email', ''));
    $mailer->Subject = 'SunMailer Test';
    $mailer->Body = print_r($mailer, TRUE);
    $mailer->Send();
  }
}

/**
 * Builds the form for one format.
 */
function _sunmailer_format_form($xid, $format) {
  // Check if the formats is disabled.
  $text_disabled = variable_get('sunmailer_disable_builtin_text', FALSE);
  $html_disabled = variable_get('sunmailer_disable_builtin_html', FALSE);
  switch ($xid) {
    case 1:
      $disabled = $text_disabled;
      break;
    case 2:
      $disabled = $html_disabled;
      break;
    default:
      $disabled = FALSE;
      break;
  }
  // Generate the title and XSL name for the format.
  $new = $xid == 'new';
  $type = $format['html'] ? 'HTML' : 'text';
  if ($new) {
    $title = 'New Format';
  }
  else {
    $xsl_name = db_result(db_query("SELECT filename FROM {files} WHERE fid = %d", $format['fid']));
    $title = $format['name'];
    if (!$format['custom']) {
      $builtin_en = t(' (built-in @type format)', array('@type' => $type));
      $builtin_dis = t(' (built-in @type format, disabled)', array('@type' => $type));
      $title .= $disabled ? $builtin_dis : $builtin_en;
    }
  }
  // Build the form.
  $form = array(
    '#type' => 'fieldset',
    '#title' => t($title),
    '#collapsible' => !$new,
    '#collapsed' => !$new,
  );
  $form['name'] = array(
    '#type' => 'textfield',
    '#title' => t('Name'),
    '#required' => !$new,
    '#default_value' => $format['name'],
  );
  $form['html'] = array(
    '#type' => 'radios',
    '#title' => t('Format Type'),
    '#options' => array(
      FALSE => t('Text'),
      TRUE => t('HTML'),
    ),
    '#default_value' => $new ? TRUE : $format['html'],
    '#disabled' => !$new,
  );
  if ($new || $format['custom']) {
    $form['file'] = array(
      '#type' => 'file',
      '#title' => t('XSL'),
      '#description' => t('XSL file used to build this format from XML content'),
    );
  }
  if (!$new) {
    $form['xsl_name'] = array(
      '#type' => 'markup',
      '#prefix' => '<p>',
      '#value' => '<strong>' . t('Current XSL File:') . "</strong> $xsl_name",
      '#suffix' => '</p>',
    );
  }
  $form['weight'] = array(
    '#type' => 'weight',
    '#title' => t('Weight'),
    '#default_value' => $new ? 0 : $format['weight'],
  );
  if (!$new) {
    $form['delete'] = array(
      '#type' => 'checkbox',
      '#title' => $format['custom'] ? t('Delete') : t('Disable'),
      '#description' => t('Users with this format will be switched to the default @type format', array('@type' => $type)),
      '#default_value' => $disabled,
    );
  }
  return $form;
}

/**
 * Generates a form for newsletter formats.
 */
function sunmailer_admin_formats() {
  $form = array();
  $form['#tree'] = TRUE;
  $form['#attributes'] = array('enctype' => 'multipart/form-data');
  // Build forms for formats.
  $formats = sunmailer_get_formats();
  foreach ($formats as $xid => $format) {
    $form[$xid] = _sunmailer_format_form($xid, $format);
  }
  $form['new'] = _sunmailer_format_form('new', array());
  // Get the list of possible default formats.
  $text_formats = array();
  $html_formats = array();
  foreach ($formats as $xid => $format) {
    if ($format['html']) {
      $html_formats[$xid] = $format['name'];
    }
    else {
      $text_formats[$xid] = $format['name'];
    }
  }
  $text_formats[0] = t('New format');
  $html_formats[0] = t('New format');
  // Build form for selecting default formats.
  $form['default'] = array(
    '#type' => 'fieldset',
    '#title' => t('Default Formats'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );
  $form['default']['html'] = array(
    '#type' => 'radios',
    '#title' => t('Default Format'),
    '#description' => t('Choose whether new users\' default format is in text or HTML'),
    '#options' => array(
      FALSE => t('Default text format'),
      TRUE => t('Default HTML format'),
    ),
    '#default_value' => variable_get('sunmailer_use_html_default', TRUE),
  );
  $form['default']['text_default'] = array(
    '#type' => 'radios',
    '#title' => t('Default Text Format'),
    '#description' => t('This format is also used as the alternate for email clients which cannot read HTML'),
    '#options' => $text_formats,
    '#default_value' => variable_get('sunmailer_text_default', 1),
  );
  $form['default']['html_default'] = array(
    '#type' => 'radios',
    '#title' => t('Default HTML Format'),
    '#options' => $html_formats,
    '#default_value' => variable_get('sunmailer_html_default', 2),
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Submit'),
  );
  return $form;
}

/**
 * Validation callback for XSLT file.
 */
function sunmailer_is_xslt_file(&$file) {
  $doc = new DOMDocument();
  return $doc->load($file->filepath) ? array() : array(t('This is not a valid XSL file.'));
}

/**
 * Validation callback for newsletter formats form.
 */
function sunmailer_admin_formats_validate($form, &$form_state) {
  $values = $form_state['values'];
  $new_name = trim($values['new']['name']);
  $validators = array(
    'sunmailer_is_xslt_file' => array(),
  );
  // Validate formats.
  $formats = sunmailer_get_formats();
  foreach ($formats as $xid => $v) {
    if (!$format['delete']) {
      $form_state['values'][$xid]['xsl'] = file_save_upload($xid, $validators, file_directory_path());
    }
  }
  if ($new_name) {
    $xslt = file_save_upload('new', $validators, file_directory_path());
    if (!$xslt) {
      form_set_error('new][file', 'XSL file required');
    }
    $form_state['values']['new']['xsl'] = $xslt;
  }
  // Get information on default formats.
  $new_html = $values['new']['html'];
  $text_default = $values['default']['text_default'];
  $html_default = $values['default']['html_default'];
  // Ensure the default format is not being deleted.
  if ($values[$text_default]['delete']) {
    form_set_error('default][text_default', t('Select a new default text format.'));
  }
  if ($values[$html_default]['delete']) {
    form_set_error('default][html_default', t('Select a new default HTML format.'));
  }
  // If the default format is a new format, ensure it exists and is the right type.
  if (!$text_default) {
    if (!$new_name) {
      form_set_error('default][text_default', t('There is no new format.'));
    }
    if ($new_html) {
      form_set_error('default][text_default', t('The default text format is not a text format.'));
    }
  }
  if (!$html_default) {
    if (!$new_name) {
      form_set_error('default][html_default', t('There is no new format.'));
    }
    if (!$new_html) {
      form_set_error('default][html_default', t('The default HTML format is not an HTML format.'));
    }
  }
}

/**
 * Submission callback for newsletter formats form.
 */
function sunmailer_admin_formats_submit($form, &$form_state) {
  $values = $form_state['values'];
  $formats = sunmailer_get_formats();
  foreach ($formats as $xid => $v) {
    $format = $values[$xid];
    // Move users to default format if current format is deleted/disabled.
    if ($format['delete']) {
      $new_xid = $formats[$xid]['html'] ? $values['default']['html_default'] : $values['default']['text_default'];
      db_query("UPDATE {sunmailer_user_format} SET xid = %d WHERE xid = %d", $new_xid, $xid);
    }
    // Update/delete a custom format.
    if ($xid > 2) {
      // Find the file for the current XSL file.
      $fid = db_result(db_query("SELECT fid FROM {sunmailer_formats} WHERE xid = %d", $xid));
      $path = db_result(db_query("SELECT filepath FROM {files} WHERE fid = %d", $fid));
      // Delete custom format.
      if ($format['delete']) {
        file_delete($path);
        db_query("DELETE FROM {sunmailer_formats} WHERE xid = %d", $xid);
      }
      // Update custom format.
      else {
        $file = $format['xsl'];
        if ($file) {
          file_set_status($file, FILE_STATUS_PERMANENT);
          db_query("DELETE FROM {files} WHERE fid = %d", $fid);
          $fid = $file->fid;
          file_delete($path);
        }
        $args = array($format['name'], $fid, $format['weight'], $xid);
        db_query("UPDATE {sunmailer_formats} SET name = '%s', fid = %d, weight = %d WHERE xid = %d", $args);
      }
    }
    // Update built-in format.
    else {
      $args = array($format['name'], $format['weight'], $xid);
      db_query("UPDATE {sunmailer_formats} SET name = '%s', weight = %d WHERE xid = %d", $args);
      if ($xid == 2) {
        variable_set('sunmailer_disable_builtin_html', $format['delete']);
      }
      else {
        variable_set('sunmailer_disable_builtin_text', $format['delete']);
      }
    }
  }
  // Check for a new format.
  $format = $values['new'];
  $name = $format['name'];
  if (trim($name)) {
    $file = $format['xsl'];
    file_set_status($file, FILE_STATUS_PERMANENT);
    $args = array($format['name'], $format['html'], $file->fid, $format['weight']);
    db_query("INSERT INTO {sunmailer_formats}(name, html, fid, weight) VALUES ('%s', %d, %d, %d)", $args);
  }
  // Update default formats.
  $defaults = $values['default'];
  variable_set('sunmailer_use_html_default', $defaults['html']);
  $text_default = $defaults['text_default'];
  if ($text_default) {
    variable_set('sunmailer_text_default', $text_default);
  }
  else {
    variable_set('sunmailer_text_default', db_result(db_query("SELECT MAX(fid) FROM {sunmailer_formats}")));
  }
  $html_default = $defaults['html_default'];
  if ($html_default) {
    variable_set('sunmailer_html_default', $html_default);
  }
  else {
    variable_set('sunmailer_html_default', db_result(db_query("SELECT MAX(fid) FROM {sunmailer_formats}")));
  }
}

/**
 * Autocomplete callback for views.
 */
function sunmailer_autocomplete_views($string = '') {
  $result = db_query_range("SELECT name FROM {views_view} WHERE name LIKE '%s%%' AND base_table = 'node'", $string, 0, 10);
  $matches = array();
  while ($match = db_result($result)) {
    $matches[$match] = $match;
  }
  drupal_json($matches);
}

/**
 * Builds the form for one section.
 */
function _sunmailer_section_form($section, $new) {
  $view = db_result(db_query("SELECT name FROM {views_view} WHERE vid = %d AND base_table = 'node'", $section['vid']));
  $form = array(
    '#type' => 'fieldset',
    '#title' => $new ? t('New Section') : $section['name'],
    '#collapsible' => !$new,
    '#collapsed' => !$new,
  );
  $form['name'] = array(
    '#type' => 'textfield',
    '#title' => t('Name'),
    '#required' => !$new,
    '#default_value' => $section['name'],
  );
  $form['view'] = array(
    '#type' => 'textfield',
    '#title' => t('View'),
    '#description' => t('View containing the content for this section'),
    '#required' => !$new,
    '#default_value' => $view,
    '#autocomplete_path' => 'sunmailer/autocomplete/views',
  );
  $form['weight'] = array(
    '#type' => 'weight',
    '#title' => t('Weight'),
    '#default_value' => $new ? 0 : $section['weight'],
  );
  if (!$new) {
    $form['delete'] = array(
      '#type' => 'checkbox',
      '#title' => t('Delete'),
      '#description' => t('Check this to delete this section'),
    );
  }
  return $form;
}

/**
 * Generates a form for newsletter sections.
 */
function sunmailer_admin_sections() {
  $form = array();
  $form['#tree'] = TRUE;
  $sections = sunmailer_get_sections();
  foreach ($sections as $sid => $section) {
    $form[$sid] = _sunmailer_section_form($section, FALSE);
  }
  $form['new'] = _sunmailer_section_form(array(), TRUE);
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Submit'),
  );
  return $form;
}

/**
 * Checks that the view exists for one section.
 */
function _sunmailer_section_validate($id, $section, $new) {
  if ($new && !trim($section['name'])) {
    return;
  }
  if (!$new && $section['delete']) {
    return;
  }
  if (!db_result(db_query("SELECT vid FROM {views_view} WHERE name = '%s' AND base_table = 'node'", $section['view']))) {
    form_set_error("$id][view", t('View does not exist or is not a node view.'));
  }
}

/**
 * Validation callback for newsletter settings form.
 */
function sunmailer_admin_sections_validate($form, &$form_state) {
  $values = $form_state['values'];
  $sections = sunmailer_get_sections();
  foreach ($sections as $sid => $v) {
    _sunmailer_section_validate($sid, $values[$sid], FALSE);
  }
  _sunmailer_section_validate('new', $values['new'], TRUE);
}

/**
 * Submission callback for newsletter settings form.
 */
function sunmailer_admin_sections_submit($form, &$form_state) {
  $values = $form_state['values'];
  $sections = sunmailer_get_sections();
  foreach ($sections as $sid => $v) {
    $section = $values[$sid];
    // Delete section, unsubscribe users from it.
    if ($section['delete']) {
      db_query("DELETE FROM {sunmailer_sections} WHERE sid = %d", $sid);
      db_query("DELETE FROM {sunmailer_user_sections} WHERE sid = %d", $sid);
    }
    // Update section.
    else {
      $vid = db_result(db_query("SELECT vid FROM {views_view} WHERE name = '%s' AND base_table = 'node'", $section['view']));
      $args = array(trim($section['name']), $vid, $section['weight'], $sid);
      db_query("UPDATE {sunmailer_sections} SET name = '%s', vid = %d, weight = %d WHERE sid = %d", $args);
    }
  }
  // Check for a new section.
  $section = $values['new'];
  $name = trim($section['name']);
  if ($name) {
    $vid = db_result(db_query("SELECT vid FROM {views_view} WHERE name = '%s' AND base_table = 'node'", $section['view']));
    $args = array($name, $vid, $section['weight']);
    db_query("INSERT INTO {sunmailer_sections}(name, vid, weight) VALUES ('%s', %d, %d)", $args);
  }
}

/**
 * Generates a form for newsletter settings.
 */
function sunmailer_admin_newsletter() {
  $form = array();
  $form['sunmailer_subject'] = array(
    '#type' => 'textfield',
    '#title' => t('Subject'),
    '#description' => t('Subject line for the newsletter'),
    '#default_value' => variable_get('sunmailer_subject', ''),
  );
  $form['sunmailer_message'] = array(
    '#type' => 'textarea',
    '#title' => t('Message'),
    '#description' => t('This message will appear at the top of the newsletter'),
    '#default_value' => variable_get('sunmailer_message', ''),
  );
  $cutoff = variable_get('sunmailer_cutoff', '');
  $form['sunmailer_cutoff'] = array(
    '#type' => 'textfield',
    '#title' => t('Cutoff Date'),
    '#description' => t('The email newsletter will exclude content published before this date'),
    '#default_value' => $cutoff ? format_date($cutoff, 'custom', 'Y-m-d H:i:s O') : '',
  );
  $form['sunmailer_submit'] = array(
    '#type' => 'submit',
    '#value' => t('Submit'),
  );
  return $form;
}

/**
 * Validation callback for newsletter settings form.
 */
function sunmailer_admin_newsletter_validate($form, &$form_state) {
  $values = $form_state['values'];
  if (!empty($values['sunmailer_cutoff']) && !strtotime($values['sunmailer_cutoff'])) {
    form_set_error('cutoff', t('Invalid date format.'));
  }
}

/**
 * Submission callback for newsletter settings form.
 */
function sunmailer_admin_newsletter_submit($form, &$form_state) {
  $values = $form_state['values'];
  if ($values['sunmailer_subject']) {
    variable_set('sunmailer_subject', $values['sunmailer_subject']);
  }
  else {
    variable_del('sunmailer_subject');
  }
  variable_set('sunmailer_message', $values['sunmailer_message']);
  variable_set('sunmailer_cutoff', strtotime($values['sunmailer_cutoff']));
}