<?php
// $Id: sunmailer.xml.inc,v 1.1.2.2 2009/12/27 00:50:47 mwacker Exp $

/**
 * @file
 * Builds the XML newsletter and the XSL to transform it.
 */

/**
 * Generates the XML for a newsletter with selected sections.
 */
function sunmailer_xml_newsletter($sids) {
  static $cached = FALSE;
  static $sections = array();
  // Build all sections once, then cache them.
  if (!$cached) {
    $cached = TRUE;
    $sections = sunmailer_get_sections();
    foreach ($sections as &$sectionn) { // http://bugs.php.net/50582
      sunmailer_get_section_nodes($sectionn);
    }
  }
  // Get XML for each section.
  $xml = '';
  foreach ($sections as $sid => $section) {
    if (in_array($sid, $sids)) {
      $xml .= sunmailer_xml_section($section);
    }
  }
  // Return nothing if no sections have content.
  if (!$xml) {
    return NULL;
  }
  // Build newsletter XML.
  $xml = sunmailer_xml_header() . $xml . sunmailer_xml_footer();
  $doc = new DOMDocument();
  $doc->loadXML($xml);
  return $doc;
}

/**
 * Retrieves the nodes for an individual section.
 */
function sunmailer_get_section_nodes(&$section) {
  $view = views_get_view($section['vid']);
  $view->execute();
  $section['nodes'] = array();
  foreach ($view->result as $row) {
    $node = node_load($row->nid);
    if ($node->created >= variable_get('sunmailer_cutoff', 0)) {
      $section['nodes'][] = $node;
    }
  }
  return $section;
}

/**
 * Generates the XML for the header.
 */
function sunmailer_xml_header() {
  static $cached = FALSE;
  static $xml;
  if ($cached) {
    return $xml;
  }
  $cached = TRUE;
  // Retrieve the header information.
  $encoding = check_plain(variable_get('sunmailer_char_set', 'utf-8'));
  $title = check_plain(variable_get('sunmailer_subject', variable_get('site_name', t('SunMailer'))));
  global $base_url;
  $date = date('l, F j, Y');
  $message = check_plain(variable_get('sunmailer_message', ''));
  // Build the header XML.
  $xml  = '<?xml version="1.0" encoding="' . $encoding .'"?>';
  $xml .= '<newsletter>';
  $xml .= "<title><![CDATA[$title]]></title>";
  $xml .= "<link><![CDATA[$base_url]]></link>";
  $xml .= "<date><![CDATA[$date]]></date>";
  if (trim($message)) {
    $xml .= "<message><![CDATA[$message]]></message>";
  }
  return $xml;
}

/**
 * Generates the XML for a section.
 */
function sunmailer_xml_section($section) {
  // Return nothing if the section has no content.
  if (!count($section['nodes'])) {
    return '';
  }
  // Retrieve the section information.
  $name = check_plain($section['name']);
  // Build the section XML.
  $xml = '<section>';
  $xml .= "<name><![CDATA[$name]]></name>";
  foreach ($section['nodes'] as $node) {
    $xml .= sunmailer_xml_content($node);
  }
  $xml .= '</section>';
  return $xml;
}

/**
 * Generates the XML for a content piece.
 */
function sunmailer_xml_content($node) {
  // Retrieve the content information.
  $title = check_plain($node->title);
  global $base_url;
  $link = url("$base_url/node/{$node->nid}");
  if ($node->uid) {
    $author = check_plain($node->name);
    $authorlink = url("$base_url/user/{$node->uid}");
  }
  $date = format_date($node->created);
  $teaser = strip_tags($node->teaser);
  // Build the content XML.
  $xml = '<content>';
  $xml .= "<title><![CDATA[$title]]></title>";
  $xml .= "<link><![CDATA[$link]]></link>";
  if ($author) {
    $xml .= "<author><![CDATA[$author]]></author>";
    $xml .= "<authorlink><![CDATA[$authorlink]]></authorlink>";
  }
  $xml .= "<date><![CDATA[$date]]></date>";
  $xml .= "<teaser><![CDATA[$teaser]]></teaser>";
  $xml .= '</content>';
  return $xml;
}

/**
 * Generates the footer for the XML.
 */
function sunmailer_xml_footer() {
  static $cached = FALSE;
  static $xml;
  if ($cached) {
    return $xml;
  }
  $cached = TRUE;
  // Retrieve the footer information.
  global $base_url;
  $notice = 'To change subscription options or unsubscribe, login to your account at ' . url("$base_url/user") . '. ';
  $notice .= 'If you forgot your password or need a new one, reset it at ' . url("$base_url/user/password") . '.';
  // Build the footer XML.
  $xml = "<notice><![CDATA[$notice]]></notice>";
  $xml .= '</newsletter>';
  return $xml;
}

/**
 * Retrieves the XSLT processor for a newsletter format.
 */
function sunmailer_get_xslt($fid) {
  $path = db_result(db_query("SELECT filepath FROM {files} WHERE fid = %d", $fid));
  $doc = new DOMDocument();
  $doc->load($path);
  $xslt = new XSLTProcessor();
  $xslt->importStyleSheet($doc);
  return $xslt;
}